#!/usr/bin/python
#
# Python script to capture ComCast Universal Caller ID messages and
# forward them to an NCID server
#
# Based on Bobby and Henry's demo code to capture, decode and print
# ComCast Caller ID messages.
#
# Rough version, assumes server is on local machine using default port
#
# TODO: More external configuration (name, pass, server address/port)
#       Parse ComCast message using XML parser?
#       Logging of actions and server messages? Own log? NCID log?
#       Replace telnetlib with functions from twisted?

from twisted.words.protocols.jabber import client, jid
from twisted.words.xish import domish
from twisted.internet import reactor
import decode as ComDecode
from time import strftime
import telnetlib
import sys, os
import logging

username = ""  # Comcast name without @comcast.net
password = ""
pidfile = "/var/run/comcast2ncid.py.pid"
NCIDserver = "localhost"
NCIDport   = 3333
logfile = "/var/log/comcast2nicd.log"
loglevel = logging.INFO


def authd(xmlstream):
    logging.info('%s authenticated with Comcast server' % username)

    global servc
    servc = telnetlib.Telnet(NCIDserver, NCIDport)
    while 1:
        resp = servc.read_until('\r\n',.5)
        if resp.startswith('200'):
            logging.info('Got OK from NCID server')
        if resp.startswith('CIDLOG'):
            logging.debug('Received CIDLOG line')
        if resp.startswith('300'):
            logging.debug('Got end of CID Log')
        if resp == '':
            logging.debug('Server done communicating')
            break


    presence = domish.Element(('jabber:client','presence'))
    xmlstream.send(presence)
    
    xmlstream.addObserver('/message',  handleMessage)
    xmlstream.addObserver('/presence', debug)
    xmlstream.addObserver('/iq',       debug)   


def debug(elem):
    logging.info('Non-message event handled')
    logging.info(elem.toXml().encode('utf-8'))
    logging.info('='*20)


def decodeCID(message):
    # I don't think I can use a proper XML parser because the last tag
    # is <callingnumber> and then </callingumber>! They don't *@&# match!
    # (I wouldn't be surprised if this is done on purpose to discourge hacks)

    print(message)  
    istart = message.find('ion>')+4
    iend   = message.find('</de')
    desc = message[istart:iend]

    istart = message.find('ame>')+4
    iend   = message.find('</callingn')
    name = message[istart:iend]

    istart = message.find('ber>')+4
    iend   = message.find('</callingu')
    number = message[istart:iend]

    return(desc, name, number)


def sendNCID(date, name, number):
    NCIDstring = 'CALL: ###DATE%s...LINE...NMBR%s...NAME%s+++' %(date, number, name)
    logging.info('Sending CID string to server')
    logging.debug(NCIDstring)
    servc.write(NCIDstring)
    while 1:
        resp = servc.read_until('\r\n',.5)
        if resp.startswith('CID:'):
            logging.info('Server responded correctly')
            logging.debug(resp)
        if resp == '':
            break
   
    
def handleMessage(elem):
    for child in elem.children:
        if child.name == "CInfo":
            logging.debug('Incoming CInfo block')
            encData = child.children[0]
            data = ComDecode.go(username, encData)
            logging.debug('Message decoded: %s' % data)
            (desc, name, number) = decodeCID(data)
            if desc.startswith('Incoming'):
                date = strftime('%m%d%H%M')
 	        logging.info('Incoming call message: %s / %s' % (name, number))
                sendNCID(date, name, number)
            else:
                logging.warning('Not Incoming Call message: %s / %s / %s' % (desc, name, number))

def daemonize (stdin='/dev/null', stdout='/dev/null', stderr='/dev/null'):

    '''This forks the current process into a daemon. The stdin, stdout, and
    stderr arguments are file names that will be opened and be used to replace
    the standard file descriptors in sys.stdin, sys.stdout, and sys.stderr.
    These arguments are optional and default to /dev/null. Note that stderr is
    opened unbuffered, so if it shares a file with stdout then interleaved
    output may not appear in the order that you expect. '''

    # Do first fork.
    try: 
        pid = os.fork()
        if pid > 0:
            # Write PID file
            logging.debug('Wrote PID %i in file %s' % (pid, pidfile))
            open(pidfile, 'w').write(str(pid) + '\n') 
            sys.exit(0)   # Exit first parent.
    except OSError, e: 
        logging.critical('Fork failed: (%d) %s\n' % (e.errno, e.strerror) )
        sys.exit(1)


    # Now I am a daemon!
    
    # Redirect standard file descriptors.
    si = open(stdin, 'r')
    so = open(stdout, 'a+')
    se = open(stderr, 'a+', 0)
    os.dup2(si.fileno(), sys.stdin.fileno())
    os.dup2(so.fileno(), sys.stdout.fileno())
    os.dup2(se.fileno(), sys.stderr.fileno())


def setLogging(file, level):
    global log
#    log = logging.getLogger('comcast2ncid')
#    handler = logging.FileHandler(file)
#    format = logging.Formatter('%(asctime)s %(levelname)5s %(message)s')
#    handler.setFormatter(format)
#    log.addHandler(handler)
    format = '%(asctime)s %(levelname)5s %(message)s'
    logging.basicConfig(level=level, filename=file, format=format)
#    log.setLevel(level)
    logging.info('='*30)
    logging.debug('Done setting Logging')


myJid = jid.JID(username + '@comcast.net/comcast')
factory = client.basicClientFactory(myJid, password)
factory.addBootstrap('//event/stream/authd',authd)
reactor.connectTCP('im.comcast.net',5222,factory)
setLogging(logfile, loglevel)
daemonize()
reactor.run()
